//
// Test application
//
// AvkonVideoOverlay
// solidcontrol.cpp
//

#include "solidcontrol.h"
#include <coemain.h>
#include <graphics/surfacemanager.h>

// Wrapper around a graphics surface
class CGraphicsSurface : public CBase
{
public:
    static CGraphicsSurface* NewL(RWindow& aWindow);
    ~CGraphicsSurface();

private:
    CGraphicsSurface(RWindow& aWindow);
    void ConstructL();

private:
    RWindow& iWindow;
    RSurfaceManager iSurfaceManager;
    TSurfaceId iSurfaceId;
    RChunk iChunk;
    bool iSurfaceBoundToWindow;
};

CGraphicsSurface* CGraphicsSurface::NewL(RWindow& aWindow)
    {
    CGraphicsSurface* self = new (ELeave) CGraphicsSurface(aWindow);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
    }

CGraphicsSurface::CGraphicsSurface(RWindow& aWindow)
    :   iWindow(aWindow)
	{
    RDebug::Printf("CGraphicsSurface::CGraphicsSurface 0x%08x window 0x%08x", this, &aWindow);
	}

void CGraphicsSurface::ConstructL()
    {
	// Connect to the surface manager
	User::LeaveIfError(iSurfaceManager.Open());

	// Create a surface
	RSurfaceManager::TSurfaceCreationAttributes attribs;
	attribs.iSize = iWindow.Size();
	attribs.iBuffers = 1;
	attribs.iPixelFormat = EUidPixelFormatARGB_8888;
	attribs.iAlignment = 1;
	attribs.iMappable = 1;
	User::LeaveIfError(iSurfaceManager.CreateSurface(attribs, iSurfaceId));
	User::LeaveIfError(iSurfaceManager.OpenSurface(iSurfaceId));

	// Get offset and stride
	RSurfaceManager::TInfoBuf infoBuf;
	RSurfaceManager::TSurfaceInfoV01& info = infoBuf();
	User::LeaveIfError(iSurfaceManager.SurfaceInfo(iSurfaceId, infoBuf));
	TInt offset = 0;
	iSurfaceManager.GetBufferOffset(iSurfaceId, 0,  offset);

	// Map surface into a chunk
	User::LeaveIfError(iSurfaceManager.MapSurface(iSurfaceId, iChunk));

	// Fill the surface with a solid colour
	TUint8* rowPtr = iChunk.Base() + offset;
	for (int i=0; i<iWindow.Size().iHeight; ++i)
		{
		TUint32* ptr = reinterpret_cast<TUint32*>(rowPtr);
		for (int j=0; j<iWindow.Size().iWidth; ++j)
			{
			*ptr++ = 0xff00ffff; // opaque cyan
			}
		rowPtr += info.iStride;
		}

	// Set the window's background surface
	User::LeaveIfError(iWindow.SetBackgroundSurface(iSurfaceId));
	}

CGraphicsSurface::~CGraphicsSurface()
	{
    RDebug::Printf("CGraphicsSurface::~CGraphicsSurface 0x%08x", this);
    iWindow.RemoveBackgroundSurface(ETrue);
	iChunk.Close();
	if (!iSurfaceId.IsNull())
        {
        iSurfaceManager.CloseSurface(iSurfaceId);
        }
	iSurfaceManager.Close();
	}


CSolidControl::CSolidControl()
    {

    }

CSolidControl::~CSolidControl()
    {
    RDebug::Printf("CSolidControl::~CSolidControl 0x%08x", this);
    delete iSurface;
    }

void CSolidControl::ConstructL(TBool aCreateSurface, CCoeControl* aParent)
    {
    iCreateSurface = aCreateSurface;
    if (aParent)
        {
        CreateWindowL(aParent);
        }
    else
        {
        CreateWindowL(CCoeEnv::Static()->RootWin());
        }
    InitComponentArrayL();
    MakeVisible(ETrue);
    ActivateL();
    RDebug::Printf("CSolidControl::ConstructL 0x%08x parent 0x%08x window 0x%08x", this, aParent, &Window());
    }

void CSolidControl::SizeChanged()
    {
    RDebug::Printf("CSolidControl::SizeChanged 0x%08x %d %d", this, Size().iWidth, Size().iHeight);
    for (CCoeControlArray::TCursor cursor = Components().Begin();
         Components().End() != cursor; cursor.Next())
        {
        cursor.Control<CCoeControl>()->SetRect(TRect(20, 20, Size().iWidth-20, Size().iHeight-20));
        }
    if (iCreateSurface)
        {
        delete iSurface;
        iSurface = 0;
        TRAP_IGNORE(iSurface = CGraphicsSurface::NewL(static_cast<RWindow&>(*DrawableWindow())));
        }
    }

void CSolidControl::Draw(const TRect& aRect) const
    {
    RDebug::Printf("CSolidControl::Draw 0x%08x surface %d", this, iCreateSurface);
    if (!iCreateSurface)
        {
        CWindowGc& gc = SystemGc();
        gc.SetBrushColor(TRgb(0, 255, 255));
        gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
        gc.Clear(aRect);
        }
    }





